/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <inttypes.h>

#include "adc.h"


static void (*adc_interrupt_handler)(void);


/*
 * A/D conversion complete interrupt handler called by avr-libc.
 * This will be enabled when adc_init(???,ADC_USE_INT) is called.
 */
ISR(ADC_vect)
{
	adc_interrupt_handler();
}


/*
 * The setter of user function
 * which will be called from A/D conversion complete interrupt handler.
 */
void
adc_set_interrupt(void (*func)(void))
{
	adc_interrupt_handler = func;
}


/*
 * Initializes A/D converter.
 * Specify one of the clock definition macros defined in adc.h to clock argment.
 *
 * If your program uses A/D conversion complete interrupt,
 * call sei() after executing this function (adc_init()).
 */
void
adc_init(uint8_t clock, int use_interrupt)
{
	clock &= (_BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0));

	if (use_interrupt) {
		/* Enables A/D conversion and complete interrupt */
		ADCSRA = (_BV(ADEN) | _BV(ADIE) | clock);
	}
	else {
		/* Enables A/D conversion */
		ADCSRA = (_BV(ADEN) | clock);
	}
}

/* End of adc.c */
